/**
 ** XMLHelper
 ** @author Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: CVS Version: $Revision: 1.2 $  Date: $Date: 1999/09/09 23:29:41 $
 ** (c) 2000 by Ben Yanbin Zhao and the Regents of the University of California
 **/

package ninja.xset;

import java.io.*;

import org.xml.sax.InputSource;
import org.w3c.dom.*;

public class XMLHelper {

  /**
   ** return a w3c Document of the parsed XML
   **/
  public static Document GetDocument( Reader xml ) {
    try {
      DOMParser parser = new DOMParser();
      InputSource is = new InputSource( xml );
      parser.parse( is );
      return ( parser.getDocument() );
    }
    catch( Exception e ) {
      // todo
      e.printStackTrace();
    }
    return null;
  }
  
  /**
   ** return a w3c Document of the parsed XML
   **/
  public static Document GetDocument( String xml ) {
    StringReader xmlreader = new StringReader( xml );
    return GetDocument( xmlreader );
  }
  
  /**
   ** return a w3c Element of the parsed XML
   **/
  public static Element GetElement( Reader xml ) {
    return GetDocument( xml ).getDocumentElement();
  }

  /**
   ** return a w3c Element of the parsed XML
   **/
  public static Element GetElement( String xml ) {
    return GetDocument( xml ).getDocumentElement();
  }

  public static void PrintDocument(Node node, PrintWriter writer, 
				   String indent) throws DOMException {
    
    if(node instanceof Element) {
      Element el = (Element)node;
      writer.print(indent+"<" + el.getTagName());
      NamedNodeMap nnm = el.getAttributes();
      int length = nnm.getLength();
      for(int i=0; i < length; i++) {
	Attr a = (Attr)nnm.item(i);
	writer.print(" " + a.getName() + "=\"" + a.getValue() + "\"");
      }
      java.util.Vector nl = SETutils.getChildren(el);
      if (nl != null) {
	length = nl.size();
	if (length == 1 && 
	    (((Node) nl.elementAt(0)).getNodeType() == Node.TEXT_NODE)) {
	  //IF Node only has one child, its text value, then print it
	      writer.print(">"+((Node) nl.elementAt(0)).getNodeValue()+
			   "</"+el.getTagName()+">\n");
	} else {
	  //Otherwise recurse on children
	  writer.print(">\n");
	  String newindent = indent + "  ";
	  for(int i = 0; i < length; i++) {
	    PrintDocument((Node) nl.elementAt(i), writer, newindent);
	  }
	  writer.print(indent+"</"+el.getTagName()+">\n");
	}
      } else {
	writer.print("/>\n");
      }
    } else if(node instanceof Text) {
      writer.print(indent+((Text)node).getData()+"\n");
    } else if(node instanceof Comment) {
      writer.print(indent + "<!--" + ((Comment)node).getData() + "-->\n"); 
    } else if( node instanceof ProcessingInstruction ) {
      // TODO 
    }
  }

  /**
   ** print a w3c xml subtree to a stream
   **/
  public static void PrintSubtree( Node n, PrintWriter writer ) 
    throws DOMException {
    PrintDocument(n, writer, "" );
    writer.flush();
  }

  /**
   ** from a w3c document, generate the XML text (e.g., to output to
   ** a file)
   **/
  public static String SubtreeToString( Node n ) {
    StringWriter writer = new StringWriter();
    PrintSubtree( n, new PrintWriter( writer ));
    writer.flush();
    return writer.getBuffer().toString();
  }

  /**
   ** print a w3c xml document to a stream
   **/
  public static void PrintDocument( Document doc, PrintWriter writer ) 
    throws DOMException {
    writer.println( "<?xml version=\"1.0\"?>" );
    PrintDocument( doc.getDocumentElement(), writer, "" );
    writer.flush();
  }

  /**
   ** from a w3c document, generate the XML text (e.g., to output to
   ** a file)
   **/
  public static String DocumentToString( Document doc ) {
    StringWriter writer = new StringWriter();
    PrintDocument( doc, new PrintWriter( writer ));
    writer.flush();
    return writer.getBuffer().toString();
  }

  public static void main(String [] argv) {
    DOMParser parser = new DOMParser();
    try {
      parser.parse("query.xml");
      Document d = parser.getDocument();
      System.out.println(DocumentToString(d));
    } catch ( Exception e) {
      e.printStackTrace(System.err);
    }
  }
}




