/**
 * @author: Ben Y. Zhao (ravenben@cs.berkeley.edu)
 ** @version: CVS Version: $Revision: 1.5 $  Date: $Date: 2000/01/06 21:19:18 $
 ** (c) 2000 by Ben Yanbin Zhao and the Regents of the University of California
 **/

package ninja.xset;

import java.io.*;
import java.util.*;
//import ninja.ispace.*;
//import ninja.xset.XSetServiceIF;

import java.rmi.*;
import java.rmi.server.*;

/**
 ** A sample client class for the XSet project
 **/
public class XSetClient {
    // iSpaceLoaderIF loader;
  XSetServiceIF XSetService;
  String hostname = null;
  String path = null;
  final String query = "<?xml version=\"1.0\"?><PRINTCAP><COLOR>YES</COLOR></PRINTCAP>";

  /**
   ** simple constructor that initializes two fields according to 
   ** cmdline arguments
   **/
  public XSetClient(String host, String xmlpath) {
    hostname = host;
    path = xmlpath;
  }

  /**
   ** Main code is here
   ** The code simply calls XSet to index the files in the XML path,
   ** and then does a query on them with a hard-coded XML query fragment.
   **/
  public void run() {
    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
    String cmd;
    try {
      while (true) {  //input loop
	cmd = br.readLine();
	if (cmd.equalsIgnoreCase("quit")) {
	  break;
	} else if (cmd.equalsIgnoreCase("init")) {
	  initialize();
	} else if (cmd.equalsIgnoreCase("query")) {
	  simplequery();
	} else if (cmd.equalsIgnoreCase("querytags")) {
	  querytags();
	} else if (cmd.equalsIgnoreCase("addfiles")) {
	  XSetService.AddFiles(path);
	} else if (cmd.equalsIgnoreCase("enumeratetags")) {
	  Vector v = new Vector();
	  String [] tagPath = new String [2];
	  tagPath[0] = "PRINTCAP";
	  tagPath[1] = "ROOM";
	  v = XSetService.EnumerateTag(tagPath);
	  System.out.println("Enumeration for room: "+v.toString());
	} else if (cmd.equalsIgnoreCase("checkpoint")) {
	  XSetService.Checkpoint();
	} else if (cmd.equalsIgnoreCase("clearall")) {
	  XSetService.ClearAll();
	} else if (cmd.equalsIgnoreCase("forcesync")) {
	  XSetService.ForceSync();
	} else if (cmd.equalsIgnoreCase("help")) {
	  System.out.println("The valid commands are:\nquit init query querytags addfiles enumeratetags checkpoint forcesync clearall");
	} else {
	  System.out.println("Invalid command. \nValid commands include the following:\nquit init query querytags addfiles enumeratetags checkpoint forcesync clearall");
	}
      }
    } catch (IOException ioe) {
      ioe.printStackTrace(System.err);
    }
  }
  private boolean initialize() {
    try {
	XSetService = (XSetServiceIF) Naming.lookup("rmi:///XSet");
      if (XSetService == null) throw new XSetException("UTOH");
    } catch (Exception e) {
      e.printStackTrace(System.err);
    }
    return true;
  }
  
  private boolean simplequery() {
    Vector results;
    try {
      results = XSetService.Query(query);
      System.out.println("Result: "+results.toString());
    } catch (java.rmi.RemoteException re) {
      re.printStackTrace(System.err);
    }
    return true;
  }
  
  private boolean querytags() {
    Vector results;
    Vector v = new Vector();
    v.addElement("NAME");
    try {
      results = XSetService.Query(query, v);
      if (results.size() == 0) System.out.println("No Matches.");
      else {
	System.out.println("Matches:");
	Hashtable ht = (Hashtable) results.elementAt(0);
	String out = (String) ((Vector) ht.get("NAME")).toString();
	System.out.println("Match: "+out);
      }
    } catch (java.rmi.RemoteException re) {
      re.printStackTrace(System.err);
    }
    return true;
  }
  
  public static void main(String argv[]) {
    if (argv.length != 2) {
      System.out.println("Wrong # of arguments!");
      System.out.println("  java ninja.xset.XSetClient hostname pathtoxmlfiles");
      System.out.println("e.g.: java ninja.xset.XSetClient joker.cs.berkeley.edu /usr/local/xmlrepository/");
      return;
    }
    XSetClient xs = new XSetClient(argv[0], argv[1]);
    xs.run();
  }
}
